import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import Tooltip from 'primereact/tooltip';
import ObjectUtils from 'primereact/components/utils/ObjectUtils';

export class Button extends Component {
	componentDidMount() {
		const { tooltip } = this.props;
		if (tooltip) {
			this.renderTooltip();
		}
	}

	componentDidUpdate(prevProps) {
		const { tooltip } = this.props;
		if (prevProps.tooltip !== tooltip) {
			if (this.tooltip) this.tooltip.updateContent(tooltip);
			else this.renderTooltip();
		}
	}

	componentWillUnmount() {
		if (this.tooltip) {
			this.tooltip.destroy();
			this.tooltip = null;
		}
	}

	renderTooltip() {
		const { tooltip, tooltipOptions } = this.props;
		this.tooltip = new Tooltip({
			target: this.element,
			content: tooltip,
			options: tooltipOptions,
		});
	}

	renderIcon() {
		const { icon, iconPos } = this.props;
		if (icon) {
			const className = classNames(icon, 'p-c', {
				'p-button-icon-left': iconPos !== 'right',
				'p-button-icon-right': iconPos === 'right',
			});
			return <span className={className} />;
		} else {
			return null;
		}
	}

	renderLabel() {
		const { label } = this.props;
		// const buttonLabel = label || 'p-btn';

		return <span className="p-button-text p-c" />;
	}

	render() {
		const { children, className, disabled, icon, iconPos, label } = this.props;
		const classNameCalc = classNames('p-button p-component', className, {
			'p-button-icon-only': icon && !label,
			'p-button-text-icon-left': icon && label && iconPos === 'left',
			'p-button-text-icon-right': icon && label && iconPos === 'right',
			'p-button-text-only': !icon && label,
			'p-disabled': disabled,
		});
		const renderIcon = this.renderIcon();
		const renderLabel = this.renderLabel();

		const buttonProps = ObjectUtils.findDiffKeys(this.props, Button.defaultProps);

		return (
			<button
				ref={el => (this.element = el)}
				{...buttonProps}
				className={classNameCalc}
				aria-label={label}
			>
				{iconPos === 'left' && renderIcon}
				{renderLabel}
				{iconPos === 'right' && renderIcon}
				{children}
			</button>
		);
	}
}

Button.defaultProps = {
	label: null,
	icon: null,
	iconPos: 'left',
	tooltip: null,
	tooltipOptions: null,
};

Button.propTypes = {
	children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node]),
	className: PropTypes.string,
	disabled: PropTypes.bool,
	icon: PropTypes.string,
	iconPos: PropTypes.string,
	label: PropTypes.string,
	tooltip: PropTypes.string,
	tooltipOptions: PropTypes.object,
};

export default Button;
