/* eslint-disable react/jsx-max-props-per-line */
/* eslint-disable no-undef */
import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import DomHandler from 'primereact/components/utils/DomHandler';

class Menu extends Component {
	componentWillUnmount() {
		this.unbindDocumentListeners();
	}

	onItemClick(event, item) {
		const { popup } = this.props;
		if (item.disabled) {
			event.preventDefault();
			return;
		}

		if (!item.url) {
			event.preventDefault();
		}

		if (item.command) {
			item.command({
				originalEvent: event,
				item,
			});
		}

		if (popup) {
			this.hide(event);
		}
	}

	onItemKeyDown(event) {
		const { buttonId } = this.props;
		const listItem = event.currentTarget.parentElement;
		const nextItem = this.findNextItem(listItem);
		const prevItem = this.findPrevItem(listItem);
		switch (event.which) {
			// down
			case 40:
				if (nextItem) {
					console.log('nextItem', nextItem.children[0]);
					nextItem.children[0].focus();
				}

				event.preventDefault();
				break;

			// up
			case 38:
				if (prevItem) {
					console.log('prevItem', prevItem.children[0]);
					prevItem.children[0].focus();
				}

				event.preventDefault();
				break;
			// tab
			case 9:
				if (event.shiftKey) {
					if (prevItem) {
						console.log('prevItem', prevItem.children[0]);
						prevItem.children[0].focus();
					}
				} else {
					if (nextItem) {
						console.log('nextItem', nextItem.children[0]);
						nextItem.children[0].focus();
					}
				}
				event.preventDefault();
				break;

			// escape
			case 27:
				$(`a#${buttonId}`).focus();
				this.hide(event);
				event.preventDefault();
				break;

			default:
				break;
		}
	}

	findNextItem(item) {
		const nextItem = item.nextElementSibling;

		if (nextItem)
			return DomHandler.hasClass(nextItem, 'p-disabled') ||
				!DomHandler.hasClass(nextItem, 'p-menuitem')
				? this.findNextItem(nextItem)
				: nextItem;
		else return null;
	}

	findPrevItem(item) {
		const prevItem = item.previousElementSibling;

		if (prevItem)
			return DomHandler.hasClass(prevItem, 'p-disabled') ||
				!DomHandler.hasClass(prevItem, 'p-menuitem')
				? this.findPrevItem(prevItem)
				: prevItem;
		else return null;
	}

	toggle(event) {
		const { popup } = this.props;
		if (popup) {
			if (this.container.offsetParent) {
				this.hide(event);
			} else {
				this.show(event);
			}
		}
	}

	show(event) {
		console.log('show');
		const { baseZIndex, onShow } = this.props;
		this.container.style.zIndex = String(baseZIndex + DomHandler.generateZIndex());
		this.container.style.display = 'block';
		const nextItem = this.container.children[0].children[0];

		setTimeout(() => {
			DomHandler.addClass(this.container, 'p-menu-overlay-visible');
			DomHandler.removeClass(this.container, 'p-menu-overlay-hidden');
		}, 1);

		DomHandler.absolutePosition(this.container, event.currentTarget);
		this.bindDocumentListeners();

		if (onShow) {
			onShow(event);
		}
		setTimeout(() => {
			console.log('show', nextItem);
			if (nextItem) {
				nextItem.children[0].focus();
			}
		}, 500);
	}

	hide(event) {
		const { onHide } = this.props;
		if (this.container) {
			DomHandler.addClass(this.container, 'p-menu-overlay-hidden');
			DomHandler.removeClass(this.container, 'p-menu-overlay-visible');

			setTimeout(() => {
				if (this.container) {
					this.container.style.display = 'none';
					DomHandler.removeClass(this.container, 'p-menu-overlay-hidden');
				}
			}, 150);
		}

		if (onHide) {
			onHide(event);
		}

		this.unbindDocumentListeners();
	}

	bindDocumentListeners() {
		if (!this.documentClickListener) {
			this.documentClickListener = event => {
				if (this.isOutsideClicked(event)) {
					this.hide();
				}
			};

			document.addEventListener('click', this.documentClickListener);
		}

		if (!this.documentResizeListener) {
			this.documentResizeListener = event => {
				if (this.container.offsetParent) {
					this.hide(event);
				}
			};

			window.addEventListener('resize', this.documentResizeListener);
		}
	}

	isOutsideClicked(event) {
		return (
			this.container &&
			!(this.container.isSameNode(event.target) || this.container.contains(event.target))
		);
	}

	unbindDocumentListeners() {
		if (this.documentClickListener) {
			document.removeEventListener('click', this.documentClickListener);
			this.documentClickListener = null;
		}

		if (this.documentResizeListener) {
			window.removeEventListener('resize', this.documentResizeListener);
			this.documentResizeListener = null;
		}
	}

	renderSubmenu(submenu, index) {
		const className = classNames('p-submenu-header', submenu.className, {
			'p-disabled': submenu.disabled,
		});
		const items = submenu.items.map((item, index) => {
			return this.renderMenuitem(item, index);
		});

		return (
			<React.Fragment key={`${submenu.label}_${index}`}>
				<li tabIndex={index} className={className} style={submenu.style} role="presentation">
					{submenu.label}
				</li>
				{items}
			</React.Fragment>
		);
	}

	renderSeparator(index) {
		return <li key={`separator_${index}`} className="p-menu-separator" role="separator" />;
	}

	renderMenuitem(item, index) {
		const className = classNames('p-menuitem', item.className, { 'p-disabled': item.disabled });
		const iconClassName = classNames(item.icon, 'p-menuitem-icon');
		const icon = item.icon ? <span className={iconClassName} /> : null;
		console.log('renderMenuitem', item.url, item.url || 'javascript:;');
		return (
			<li key={`${item.label}_${index}`} className={className} style={item.style} role="menuitem">
				<a
					href={item.url ? item.url : 'javascript:;'}
					className="p-menuitem-link test2"
					role="menuitem"
					target={item.target}
					tabIndex={index + 1}
					onClick={e => this.onItemClick(e, item)}
					onKeyDown={e => this.onItemKeyDown(e, item)}
				>
					{icon}
					<span className="p-menuitem-text">{item.label}</span>
				</a>
			</li>
		);
	}

	renderItem(item, index) {
		if (item.separator) {
			return this.renderSeparator(index);
		} else {
			if (item.items) return this.renderSubmenu(item, index);
			else return this.renderMenuitem(item, index);
		}
	}

	renderMenu() {
		const { model } = this.props;
		return model.map((item, index) => {
			return this.renderItem(item, index);
		});
	}

	renderElement() {
		const { className, id, model, popup, style } = this.props;
		if (model) {
			const classNameCalc = classNames('p-menu p-component', className, {
				'p-menu-dynamic p-menu-overlay': popup,
			});
			const menuitems = this.renderMenu();

			return (
				<div id={id} className={classNameCalc} style={style} ref={el => (this.container = el)}>
					<ul className="p-menu-list p-reset" role="menu" tabIndex="0">
						{menuitems}
					</ul>
				</div>
			);
		} else {
			return null;
		}
	}

	render() {
		const { appendTo } = this.props;
		const element = this.renderElement();
		if (appendTo) {
			return ReactDOM.createPortal(element, appendTo);
		} else {
			return element;
		}
	}
}

Menu.defaultProps = {
	id: null,
	model: null,
	popup: false,
	style: null,
	className: null,
	autoZIndex: true,
	baseZIndex: 0,
	appendTo: null,
	onShow: null,
	onHide: null,
};

Menu.propTypes = {
	appendTo: PropTypes.any,
	autoZIndex: PropTypes.bool,
	baseZIndex: PropTypes.number,
	buttonId: PropTypes.string,
	className: PropTypes.string,
	id: PropTypes.string,
	model: PropTypes.array,
	onHide: PropTypes.func,
	onShow: PropTypes.func,
	popup: PropTypes.bool,
	style: PropTypes.object,
};

export default Menu;
