/* eslint-disable react/jsx-handler-names */
/* eslint-disable react/destructuring-assignment */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

export class SelectButtonItem extends Component {
	constructor(props) {
		super(props);
		this.state = {};
		this.onClick = this.onClick.bind(this);
		this.onFocus = this.onFocus.bind(this);
		this.onBlur = this.onBlur.bind(this);
		this.onKeyDown = this.onKeyDown.bind(this);
	}

	componentDidUpdate() {
		this.input.checked = this.props.selected;
	}

	onClick(event) {
		if (this.props.onClick) {
			this.props.onClick({
				originalEvent: event,
				option: this.props.option,
			});

			this.input.focus();
		}
	}

	onFocus() {
		this.setState({ focused: true });
	}

	onBlur() {
		this.setState({ focused: false });
	}

	onKeyDown(event) {
		if (event.key === 'Enter') {
			this.onClick(event);
			event.preventDefault();
		}
	}

	render() {
		const className = classNames('p-button p-component p-button-text-only', this.props.className, {
			'p-highlight': this.props.selected,
			'p-disabled': this.props.disabled,
			'p-focus': this.state.focused,
		});

		return (
			<div
				ref={el => (this.el = el)}
				className={className}
				onClick={this.onClick}
				role="button"
				aria-pressed={this.props.selected}
				aria-labelledby={this.props.ariaLabelledBy}
			>
				<span className="p-button-text p-c">{this.props.label}</span>
				<div className="p-hidden-accessible">
					<input
						ref={el => (this.input = el)}
						type="checkbox"
						defaultChecked={this.props.selected}
						onFocus={this.onFocus}
						onBlur={this.onBlur}
						onKeyDown={this.onKeyDown}
						tabIndex={this.props.tabIndex}
						disabled={this.props.disabled}
						value={this.props.label}
						required={this.props.required}
						aria-required={this.props.required}
						aria-label={`${this.props.ariaLabel} - ${this.props.label}`}
						aria-describedby={this.props.ariaDescribedBy}
					/>
				</div>
			</div>
		);
	}
}

SelectButtonItem.defaultProps = {
	option: null,
	label: null,
	className: null,
	selected: null,
	tabIndex: null,
	ariaLabelledBy: null,
	onClick: null,
};

SelectButtonItem.propTypes = {
	ariaDescribedBy: PropTypes.string,
	ariaLabel: PropTypes.string,
	ariaLabelledBy: PropTypes.string,
	className: PropTypes.string,
	disabled: PropTypes.bool,
	label: PropTypes.string,
	name: PropTypes.string,
	onClick: PropTypes.func,
	option: PropTypes.object,
	required: PropTypes.bool,
	selected: PropTypes.bool,
	tabIndex: PropTypes.number,
};

export default SelectButtonItem;
